#!/usr/bin/env python

"""
Handlers for a person for whom scheduling is performed.
"""

from email.mime.text import MIMEText
from imiptools.config import MANAGER_PATH, MANAGER_URL
from imiptools.content import Handler, to_part
from socket import gethostname
from vCalendar import to_node

def get_manager_url():
    url_base = MANAGER_URL or "http://%s/" % gethostname()
    return "%s/%s" % (url_base.rstrip("/"), MANAGER_PATH.lstrip("/"))

class PersonHandler(Handler):

    "Handling mechanisms specific to people."

    def _record_and_deliver(self, objtype, queue=False):

        oa = self.require_organiser_and_attendees()
        if not oa:
            return False

        (organiser, organiser_attr), attendees = oa

        # Process each attendee separately.

        for attendee, attendee_attr in attendees.items():

            if not self.have_new_object(attendee, objtype):
                continue

            # Store the object and queue any request.

            self.store.set_event(attendee, self.uid, to_node(
                {objtype : [(self.details, {})]}
                ))

            if queue:
                self.store.queue_request(attendee, self.uid)

        return True

class Event(PersonHandler):

    "An event handler."

    def add(self):

        # NOTE: Queue a suggested modification to any active event.

        # The message is now wrapped and passed on to the recipient.

        return "ADD", MIMEText("An addition to an event has been received.")

    def cancel(self):

        # NOTE: Queue a suggested modification to any active event.

        # The message is now wrapped and passed on to the recipient.

        return "CANCEL", MIMEText("A cancellation has been received.")

    def counter(self):

        # NOTE: Queue a suggested modification to any active event.

        # The message is now wrapped and passed on to the recipient.

        return "COUNTER", MIMEText("A counter proposal has been received.")

    def declinecounter(self):

        # NOTE: Queue a suggested modification to any active event.

        # The message is now wrapped and passed on to the recipient.

        return "DECLINECOUNTER", MIMEText("A declining counter proposal has been received.")

    def publish(self):

        # NOTE: Register details of any relevant event.

        # The message is now wrapped and passed on to the recipient.

        return "PUBLISH", MIMEText("Details of an event have been received.")

    def refresh(self):

        # NOTE: Update details of any active event.

        # The message is now wrapped and passed on to the recipient.

        return "REFRESH", MIMEText("An event update has been received.")

    def reply(self):

        "Record replies and notify the recipient."

        self._record_and_deliver("VEVENT", False)

        # The message is now wrapped and passed on to the recipient.

        return "REPLY", MIMEText("A reply has been received.")

    def request(self):

        "Hold requests and notify the recipient."

        self._record_and_deliver("VEVENT", True)

        # The message is now wrapped and passed on to the recipient.

        url = "%s/%s" % (get_manager_url().rstrip("/"), self.uid)
        return "REQUEST", MIMEText("A request has been queued and can be viewed here: %s" % url)

class Freebusy(PersonHandler):

    "A free/busy handler."

    def publish(self):

        # NOTE: Register free/busy information.

        # The message is now wrapped and passed on to the recipient.

        return "PUBLISH", MIMEText("Details of a contact's availability have been received.")

    def reply(self):

        "Record replies and notify the recipient."

        self._record_and_deliver("VFREEBUSY", False)

        # The message is now wrapped and passed on to the recipient.

        return "REPLY", MIMEText("A reply has been received.")

    def request(self):

        """
        Respond to a request by preparing a reply containing free/busy
        information for each indicated attendee.
        """

        # NOTE: This is currently the same as the resource handler but should be
        # NOTE: subject to policy/preferences.

        oa = self.require_organiser_and_attendees()
        if not oa:
            return None

        (organiser, organiser_attr), attendees = oa

        # Construct an appropriate fragment.

        calendar = []
        cwrite = calendar.append

        # Get the details for each attendee.

        for attendee, attendee_attr in attendees.items():
            freebusy = self.store.get_freebusy(attendee)

            record = []
            rwrite = record.append

            rwrite(("ORGANIZER", organiser_attr, organiser))
            rwrite(("ATTENDEE", attendee_attr, attendee))
            rwrite(("UID", {}, self.uid))

            if freebusy:
                for start, end, uid in freebusy:
                    rwrite(("FREEBUSY", {"FBTYPE" : "BUSY"}, [start, end]))

            cwrite(("VFREEBUSY", {}, record))

        # Return the reply.

        return "REPLY", to_part("REPLY", calendar)

class Journal(PersonHandler):

    "A journal entry handler."

    def add(self):

        # NOTE: Queue a suggested modification to any active entry.

        # The message is now wrapped and passed on to the recipient.

        return "ADD", MIMEText("An addition to a journal entry has been received.")

    def cancel(self):

        # NOTE: Queue a suggested modification to any active entry.

        # The message is now wrapped and passed on to the recipient.

        return "CANCEL", MIMEText("A cancellation has been received.")

    def publish(self):

        # NOTE: Register details of any relevant entry.

        # The message is now wrapped and passed on to the recipient.

        return "PUBLISH", MIMEText("Details of a journal entry have been received.")

class Todo(PersonHandler):

    "A to-do item handler."

    def add(self):

        # NOTE: Queue a suggested modification to any active item.

        # The message is now wrapped and passed on to the recipient.

        return "ADD", MIMEText("An addition to an item has been received.")

    def cancel(self):

        # NOTE: Queue a suggested modification to any active item.

        # The message is now wrapped and passed on to the recipient.

        return "CANCEL", MIMEText("A cancellation has been received.")

    def counter(self):

        # NOTE: Queue a suggested modification to any active item.

        # The message is now wrapped and passed on to the recipient.

        return "COUNTER", MIMEText("A counter proposal has been received.")

    def declinecounter(self):

        # NOTE: Queue a suggested modification to any active item.

        # The message is now wrapped and passed on to the recipient.

        return "DECLINECOUNTER", MIMEText("A declining counter proposal has been received.")

    def publish(self):

        # NOTE: Register details of any relevant item.

        # The message is now wrapped and passed on to the recipient.

        return "PUBLISH", MIMEText("Details of an item have been received.")

    def refresh(self):

        # NOTE: Update details of any active item.

        # The message is now wrapped and passed on to the recipient.

        return "REFRESH", MIMEText("An item update has been received.")

    def reply(self):

        "Record replies and notify the recipient."

        self._record_and_deliver("VTODO", False)

        # The message is now wrapped and passed on to the recipient.

        return "REPLY", MIMEText("A reply has been received.")

    def request(self):

        "Hold requests and notify the recipient."

        self._record_and_deliver("VTODO", True)

        # The message is now wrapped and passed on to the recipient.

        return "REQUEST", MIMEText("A request has been queued.")

# Handler registry.

handlers = [
    ("VFREEBUSY",   Freebusy),
    ("VEVENT",      Event),
    ("VTODO",       Todo),
    ("VJOURNAL",    Journal),
    ]

# vim: tabstop=4 expandtab shiftwidth=4
